package henkan.dsl.ide

import io.undertow.Undertow
import static io.undertow.Handlers.websocket
import static io.undertow.Handlers.path
import io.undertow.websockets.WebSocketConnectionCallback
import io.undertow.websockets.spi.WebSocketHttpExchange
import io.undertow.websockets.core.WebSocketChannel
import io.undertow.websockets.core.AbstractReceiveListener
import io.undertow.websockets.core.BufferedTextMessage
import org.eclipse.lsp4j.jsonrpc.Launcher
import java.util.Map
import org.eclipse.lsp4j.jsonrpc.json.JsonRpcMethod
import java.util.LinkedHashMap
import org.eclipse.lsp4j.jsonrpc.services.ServiceEndpoints
import org.eclipse.lsp4j.jsonrpc.json.JsonRpcMethodProvider
import org.eclipse.xtext.xbase.lib.Functions.Function1
import org.eclipse.lsp4j.jsonrpc.MessageConsumer
import org.eclipse.lsp4j.jsonrpc.json.StreamMessageConsumer
import org.eclipse.lsp4j.jsonrpc.json.MessageJsonHandler
import org.eclipse.lsp4j.jsonrpc.RemoteEndpoint
import org.eclipse.lsp4j.jsonrpc.json.StreamMessageProducer
import org.eclipse.lsp4j.jsonrpc.json.ConcurrentMessageProcessor
import java.util.concurrent.ExecutorService
import java.io.InputStream
import java.io.OutputStream

class UndertowExample {
	def static void main(String[] args) {
		val builder  = Undertow.builder.addHttpListener(4389,"localhost").handler = path.addPrefixPath("/",websocket(new WebSocketConnectionCallback(){
			
			override onConnect(WebSocketHttpExchange exchange, WebSocketChannel channel) {
				
				channel.receiveSetter.set(new AbstractReceiveListener(){
					override onFullTextMessage(WebSocketChannel channel,BufferedTextMessage message){
						println('''data received: «message.data»''')
					}
				})
				channel.resumeReceives
			}
			
		}))
		
		builder.build.start
	}
	
	def static <T> Launcher<T> createIoLauncher(Object localService, Class<T> remoteInterface, InputStream in, OutputStream out, ExecutorService executorService, (MessageConsumer) => MessageConsumer wrapper) {
		val Map<String, JsonRpcMethod> supportedMethods = new LinkedHashMap<String, JsonRpcMethod>();
		supportedMethods.putAll(ServiceEndpoints.getSupportedMethods(remoteInterface));
		
		if (localService instanceof JsonRpcMethodProvider) {
			val rpcMethodProvider = localService as JsonRpcMethodProvider;
			supportedMethods.putAll(rpcMethodProvider.supportedMethods());
		} else {
			supportedMethods.putAll(ServiceEndpoints.getSupportedMethods(localService.getClass()));
		}
		
		val jsonHandler = new MessageJsonHandler(supportedMethods);
		val outGoingMessageStream = wrapper.apply(new StreamMessageConsumer(out, jsonHandler));
		val serverEndpoint = new RemoteEndpoint(outGoingMessageStream, ServiceEndpoints.toEndpoint(localService));
		jsonHandler.setMethodProvider(serverEndpoint);
		// wrap incoming message stream
		val messageConsumer = wrapper.apply(serverEndpoint);
		val reader = new StreamMessageProducer(in, jsonHandler);
		
		val rmtProxt = ServiceEndpoints.toServiceObject(serverEndpoint, remoteInterface);
		
		return new Launcher<T> () {

			override getRemoteProxy() {
				return rmtProxt;
			}
			
			override startListening() {
				return ConcurrentMessageProcessor.startProcessing(reader, messageConsumer, executorService);
			}
			
		};
	}
}