package henkan.dsl.ide;

import io.undertow.Handlers;
import io.undertow.Undertow;
import io.undertow.websockets.WebSocketConnectionCallback;
import io.undertow.websockets.core.AbstractReceiveListener;
import io.undertow.websockets.core.BufferedTextMessage;
import io.undertow.websockets.core.WebSocketChannel;
import io.undertow.websockets.spi.WebSocketHttpExchange;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import org.eclipse.lsp4j.jsonrpc.Endpoint;
import org.eclipse.lsp4j.jsonrpc.Launcher;
import org.eclipse.lsp4j.jsonrpc.MessageConsumer;
import org.eclipse.lsp4j.jsonrpc.RemoteEndpoint;
import org.eclipse.lsp4j.jsonrpc.json.ConcurrentMessageProcessor;
import org.eclipse.lsp4j.jsonrpc.json.JsonRpcMethod;
import org.eclipse.lsp4j.jsonrpc.json.JsonRpcMethodProvider;
import org.eclipse.lsp4j.jsonrpc.json.MessageJsonHandler;
import org.eclipse.lsp4j.jsonrpc.json.StreamMessageConsumer;
import org.eclipse.lsp4j.jsonrpc.json.StreamMessageProducer;
import org.eclipse.lsp4j.jsonrpc.services.ServiceEndpoints;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.xnio.ChannelListener;

@SuppressWarnings("all")
public class UndertowExample {
  public static void main(final String[] args) {
    Undertow.Builder _addHttpListener = Undertow.builder().addHttpListener(4389, "localhost");
    final Undertow.Builder builder = _addHttpListener.setHandler(Handlers.path().addPrefixPath("/", Handlers.websocket(new WebSocketConnectionCallback() {
      public void onConnect(final WebSocketHttpExchange exchange, final WebSocketChannel channel) {
        ChannelListener.Setter<WebSocketChannel> _receiveSetter = channel.getReceiveSetter();
        _receiveSetter.set(new AbstractReceiveListener() {
          public void onFullTextMessage(final WebSocketChannel channel, final BufferedTextMessage message) {
            StringConcatenation _builder = new StringConcatenation();
            _builder.append("data received: ");
            String _data = message.getData();
            _builder.append(_data);
            InputOutput.<String>println(_builder.toString());
          }
        });
        channel.resumeReceives();
      }
    })));
    builder.build().start();
  }
  
  public static <T extends Object> Launcher<T> createIoLauncher(final Object localService, final Class<T> remoteInterface, final InputStream in, final OutputStream out, final ExecutorService executorService, final Function1<? super MessageConsumer, ? extends MessageConsumer> wrapper) {
    final Map<String, JsonRpcMethod> supportedMethods = new LinkedHashMap<String, JsonRpcMethod>();
    supportedMethods.putAll(ServiceEndpoints.getSupportedMethods(remoteInterface));
    if ((localService instanceof JsonRpcMethodProvider)) {
      final JsonRpcMethodProvider rpcMethodProvider = ((JsonRpcMethodProvider) localService);
      supportedMethods.putAll(rpcMethodProvider.supportedMethods());
    } else {
      supportedMethods.putAll(ServiceEndpoints.getSupportedMethods(localService.getClass()));
    }
    final MessageJsonHandler jsonHandler = new MessageJsonHandler(supportedMethods);
    StreamMessageConsumer _streamMessageConsumer = new StreamMessageConsumer(out, jsonHandler);
    final MessageConsumer outGoingMessageStream = wrapper.apply(_streamMessageConsumer);
    Endpoint _endpoint = ServiceEndpoints.toEndpoint(localService);
    final RemoteEndpoint serverEndpoint = new RemoteEndpoint(outGoingMessageStream, _endpoint);
    jsonHandler.setMethodProvider(serverEndpoint);
    final MessageConsumer messageConsumer = wrapper.apply(serverEndpoint);
    final StreamMessageProducer reader = new StreamMessageProducer(in, jsonHandler);
    final T rmtProxt = ServiceEndpoints.<T>toServiceObject(serverEndpoint, remoteInterface);
    return new Launcher<T>() {
      public T getRemoteProxy() {
        return rmtProxt;
      }
      
      public Future<?> startListening() {
        return ConcurrentMessageProcessor.startProcessing(reader, messageConsumer, executorService);
      }
    };
  }
}
