/*
 * generated by Xtext 2.11.0
 */
package henkan.dsl.jvmmodel

import com.google.inject.Inject
import org.eclipse.xtext.xbase.jvmmodel.AbstractModelInferrer
import org.eclipse.xtext.xbase.jvmmodel.IJvmDeclaredTypeAcceptor
import org.eclipse.xtext.xbase.jvmmodel.JvmTypesBuilder
import org.eclipse.xtext.util.internal.Log
import henkan.dsl.hDsl.OperatorDescription
import henkan.dsl.repo.SourceTypeRepo
import org.eclipse.emf.ecore.EObject
import henkan.dsl.hDsl.SourceDescription
import java.util.Iterator
import org.eclipse.xtext.common.types.JvmField
import org.eclipse.xtext.common.types.JvmTypeReference
import org.eclipse.xtext.common.types.TypesFactory
import org.eclipse.xtext.common.types.JvmVisibility
import henkan.dsl.hDsl.EmitExpression
import org.eclipse.xtext.xbase.XExpression
import henkan.dsl.hDsl.ExpressionColumn
import org.eclipse.xtext.common.types.JvmDeclaredType
import org.eclipse.xtext.xbase.XAbstractFeatureCall
import henkan.dsl.lib.Tuple2
import henkan.dsl.hDsl.RegExColumn
import java.util.regex.Pattern
import henkan.dsl.lib.HenkanRow
import org.apache.flink.util.Collector
import henkan.dsl.hDsl.HenkanClosure

/**
 * <p>Infers a JVM model from the source model.</p> 
 * 
 * <p>The JVM model should contain all elements that would appear in the Java code 
 * which is generated from the source model. Other models link against the JVM model rather than the source model.</p>     
 */
@Log class HDslJvmModelInferrer extends AbstractModelInferrer {

	/**
	 * convenience API to build and initialize JVM types and their members.
	 */
	@Inject extension JvmTypesBuilder
	@Inject SourceTypeRepo repo
	@Inject TypesFactory typesFactory;
	//@Inject IBatchTypeResolver batchTypeResolver;

	/**
	 * The dispatch method {@code infer} is called for each instance of the
	 * given element's type that is contained in a resource.
	 * 
	 * @param element
	 *            the model to create one or more
	 *            {@link org.eclipse.xtext.common.types.JvmDeclaredType declared
	 *            types} from.
	 * @param acceptor
	 *            each created
	 *            {@link org.eclipse.xtext.common.types.JvmDeclaredType type}
	 *            without a container should be passed to the acceptor in order
	 *            get attached to the current resource. The acceptor's
	 *            {@link IJvmDeclaredTypeAcceptor#accept(org.eclipse.xtext.common.types.JvmDeclaredType)
	 *            accept(..)} method takes the constructed empty type for the
	 *            pre-indexing phase. This one is further initialized in the
	 *            indexing phase using the lambda you pass as the last argument.
	 * @param isPreIndexingPhase
	 *            whether the method is called in a pre-indexing phase, i.e.
	 *            when the global index is not yet fully updated. You must not
	 *            rely on linking using the index if isPreIndexingPhase is
	 *            <code>true</code>.
	 */
	def dispatch void infer(HenkanClosure element, IJvmDeclaredTypeAcceptor acceptor, boolean isPreIndexingPhase) {
		val operator = getOperator(element)
		val className = operator.name
		acceptor.accept(element.toClass('''«className»Function''')) [
			members += element.toMethod('main', typeRef(Void.TYPE)) [
				// add a parameter String[] args
				parameters += element.toParameter("rows", typeRef(Iterator, typeRef('''«operator.source.name»Row''')))
				parameters += element.toParameter("collector", typeRef(Collector, typeRef(HenkanRow)))
				// Associate the model with the body of the main method
				body = element
			]
		]
		for (EObject child : element.eContents()) {
			infer(child, acceptor, isPreIndexingPhase);
		}
	}

	def dispatch void infer(SourceDescription source, IJvmDeclaredTypeAcceptor acceptor, boolean isPreIndexingPhase) {
		val sourceID = Integer.valueOf(source.id)
		val columns = repo.load(sourceID)
		acceptor.accept(source.toClass('''«source.name»Row''')) [
			superTypes += typeRef(HenkanRow)
			for (a : columns) {
				val field=source.toField(a.columnName, a.columnType.typeRef)
				field.visibility = JvmVisibility.PUBLIC
				members += field
			}
		]
	}

	def dispatch void infer(EmitExpression emit, IJvmDeclaredTypeAcceptor acceptor, boolean isPreIndexingPhase) {
		val operator = getOperator(emit)
		LOG.info('''emit column count:«emit.columns.size» operator name «operator.name»''')
		acceptor.accept(emit.toClass('''«operator.name»Row''')) [
			superTypes += typeRef(HenkanRow)
			if (!isPreIndexingPhase) {
				for (c : emit.columns) {
					if (c instanceof ExpressionColumn) {
						val name = if (c.name !== null)
								c.name
							else {
								(c.xpr as XAbstractFeatureCall).feature.simpleName
							}
						LOG.info('''operator name «operator.name» name:«name»''')
						val inferred = c.xpr.inferredType
						val field = c.xpr.toField(name, inferred)
						field.visibility = JvmVisibility.PUBLIC
						members += field
					} else {
						val p = Pattern.compile((c as RegExColumn).regex)

						val filteredMembers = (typeRef('''«operator.source.name»Row''') as JvmDeclaredType).
							declaredFields.map [
								new Tuple2(it.simpleName, it)
							].filter [
								p.matcher(it.first).matches
							].map [
								c.toField(it.first, it.second.type)
							]
						members += filteredMembers
					}
				}

			}
		]
	}

	public static def OperatorDescription getOperator(XExpression element) {
		var current = element.eContainer
		while (current !== null && !(current instanceof OperatorDescription)) {
			current = current.eContainer
		}
		return current as OperatorDescription
	}

	def JvmField myToFiedfdld( /* @Nullable */ EObject sourceElement, /* @Nullable */ String name, /* @Nullable */ JvmTypeReference typeRef) {
		if (sourceElement === null || name === null)
			return null;
		val result = typesFactory.createJvmField();
		result.setSimpleName(name);
		result.setVisibility(JvmVisibility.PUBLIC);
		result.setType(cloneWithProxies(typeRef));
		associate(sourceElement, result);
		return result;
	}
}
