package henkan.dsl.jvmmodel;

import henkan.dsl.hDsl.EmitExpression;
import henkan.dsl.hDsl.ExpressionColumn;
import java.util.Arrays;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.util.internal.Log;
import org.eclipse.xtext.xbase.XAbstractFeatureCall;
import org.eclipse.xtext.xbase.XAssignment;
import org.eclipse.xtext.xbase.XBasicForLoopExpression;
import org.eclipse.xtext.xbase.XBlockExpression;
import org.eclipse.xtext.xbase.XBooleanLiteral;
import org.eclipse.xtext.xbase.XCastedExpression;
import org.eclipse.xtext.xbase.XClosure;
import org.eclipse.xtext.xbase.XConstructorCall;
import org.eclipse.xtext.xbase.XDoWhileExpression;
import org.eclipse.xtext.xbase.XExpression;
import org.eclipse.xtext.xbase.XForLoopExpression;
import org.eclipse.xtext.xbase.XIfExpression;
import org.eclipse.xtext.xbase.XInstanceOfExpression;
import org.eclipse.xtext.xbase.XListLiteral;
import org.eclipse.xtext.xbase.XNullLiteral;
import org.eclipse.xtext.xbase.XNumberLiteral;
import org.eclipse.xtext.xbase.XReturnExpression;
import org.eclipse.xtext.xbase.XSetLiteral;
import org.eclipse.xtext.xbase.XStringLiteral;
import org.eclipse.xtext.xbase.XSwitchExpression;
import org.eclipse.xtext.xbase.XSynchronizedExpression;
import org.eclipse.xtext.xbase.XThrowExpression;
import org.eclipse.xtext.xbase.XTryCatchFinallyExpression;
import org.eclipse.xtext.xbase.XTypeLiteral;
import org.eclipse.xtext.xbase.XVariableDeclaration;
import org.eclipse.xtext.xbase.XWhileExpression;
import org.eclipse.xtext.xbase.typesystem.computation.ITypeComputationResult;
import org.eclipse.xtext.xbase.typesystem.computation.ITypeComputationState;
import org.eclipse.xtext.xbase.typesystem.computation.XbaseTypeComputer;

@Log
@SuppressWarnings("all")
public class HenkanTypeComputer extends XbaseTypeComputer {
  protected void _computeTypes(final EmitExpression emit, final ITypeComputationState state) {
    EList<XExpression> _columns = emit.getColumns();
    for (final XExpression column : _columns) {
      if ((column instanceof ExpressionColumn)) {
        HenkanTypeComputer.LOG.info(("expr column" + column));
        final ITypeComputationResult result = state.withNonVoidExpectation().computeTypes(column);
        HenkanTypeComputer.LOG.info(result);
      }
    }
    HenkanTypeComputer.LOG.info("called");
    state.acceptActualType(this.getPrimitiveVoid(state));
  }
  
  protected void _computeTypes(final ExpressionColumn column, final ITypeComputationState state) {
    HenkanTypeComputer.LOG.info("expression column compute types");
    final ITypeComputationResult result = state.withNonVoidExpectation().computeTypes(column.getXpr());
    state.acceptActualType(result.getActualExpressionType());
  }
  
  public void computeTypes(final XExpression emit, final ITypeComputationState state) {
    if (emit instanceof XAssignment) {
      _computeTypes((XAssignment)emit, state);
      return;
    } else if (emit instanceof XDoWhileExpression) {
      _computeTypes((XDoWhileExpression)emit, state);
      return;
    } else if (emit instanceof XListLiteral) {
      _computeTypes((XListLiteral)emit, state);
      return;
    } else if (emit instanceof XSetLiteral) {
      _computeTypes((XSetLiteral)emit, state);
      return;
    } else if (emit instanceof XWhileExpression) {
      _computeTypes((XWhileExpression)emit, state);
      return;
    } else if (emit instanceof EmitExpression) {
      _computeTypes((EmitExpression)emit, state);
      return;
    } else if (emit instanceof ExpressionColumn) {
      _computeTypes((ExpressionColumn)emit, state);
      return;
    } else if (emit instanceof XAbstractFeatureCall) {
      _computeTypes((XAbstractFeatureCall)emit, state);
      return;
    } else if (emit instanceof XBasicForLoopExpression) {
      _computeTypes((XBasicForLoopExpression)emit, state);
      return;
    } else if (emit instanceof XBlockExpression) {
      _computeTypes((XBlockExpression)emit, state);
      return;
    } else if (emit instanceof XBooleanLiteral) {
      _computeTypes((XBooleanLiteral)emit, state);
      return;
    } else if (emit instanceof XCastedExpression) {
      _computeTypes((XCastedExpression)emit, state);
      return;
    } else if (emit instanceof XClosure) {
      _computeTypes((XClosure)emit, state);
      return;
    } else if (emit instanceof XConstructorCall) {
      _computeTypes((XConstructorCall)emit, state);
      return;
    } else if (emit instanceof XForLoopExpression) {
      _computeTypes((XForLoopExpression)emit, state);
      return;
    } else if (emit instanceof XIfExpression) {
      _computeTypes((XIfExpression)emit, state);
      return;
    } else if (emit instanceof XInstanceOfExpression) {
      _computeTypes((XInstanceOfExpression)emit, state);
      return;
    } else if (emit instanceof XNullLiteral) {
      _computeTypes((XNullLiteral)emit, state);
      return;
    } else if (emit instanceof XNumberLiteral) {
      _computeTypes((XNumberLiteral)emit, state);
      return;
    } else if (emit instanceof XReturnExpression) {
      _computeTypes((XReturnExpression)emit, state);
      return;
    } else if (emit instanceof XStringLiteral) {
      _computeTypes((XStringLiteral)emit, state);
      return;
    } else if (emit instanceof XSwitchExpression) {
      _computeTypes((XSwitchExpression)emit, state);
      return;
    } else if (emit instanceof XSynchronizedExpression) {
      _computeTypes((XSynchronizedExpression)emit, state);
      return;
    } else if (emit instanceof XThrowExpression) {
      _computeTypes((XThrowExpression)emit, state);
      return;
    } else if (emit instanceof XTryCatchFinallyExpression) {
      _computeTypes((XTryCatchFinallyExpression)emit, state);
      return;
    } else if (emit instanceof XTypeLiteral) {
      _computeTypes((XTypeLiteral)emit, state);
      return;
    } else if (emit instanceof XVariableDeclaration) {
      _computeTypes((XVariableDeclaration)emit, state);
      return;
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(emit, state).toString());
    }
  }
  
  private final static Logger LOG = Logger.getLogger(HenkanTypeComputer.class);
}
