'use strict';

import * as net from 'net';

import {Trace} from 'vscode-jsonrpc';
import { workspace, ExtensionContext } from 'vscode';
import { ErrorAction,CloseAction,RevealOutputChannelOn,LanguageClient, LanguageClientOptions, StreamInfo } from 'vscode-languageclient';

export function activate(context: ExtensionContext) {
    // The server is a started as a separate app and listens on port 5007
    var websocket = require("websocket-stream")
    
    let connectionInfo = {
        port: 4389,
        host:"localhost"
    };
    let serverOptions = () => {
        // Connect to language server via socket
        console.log("preparing connection")
        let socket = websocket("ws://localhost:4389")
        let result: StreamInfo = {
            writer: socket,
            reader: socket
        };
        console.log("connection established")
        return Promise.resolve(result);
    };
    
    let clientOptions: LanguageClientOptions = {
        documentSelector: ['hdsl'],
        synchronize: {
            configurationSection: 'hdslserver',
            fileEvents: workspace.createFileSystemWatcher('**/*.hl')
        },
        revealOutputChannelOn:RevealOutputChannelOn.Info,
		errorHandler: {
                error: (e,m,c) => {
					console.log(`error:${e} message:${m} count:${c}`)
					//console.log(`got error`)
					return ErrorAction.Continue
				},
                closed: () => CloseAction.DoNotRestart
        },
		initializationFailedHandler:() => {return false;}
    };
    
    // Create the language client and start the client.
    let lc = new LanguageClient('HDSLLanguageServer','HDsl Language Server', serverOptions, clientOptions);
    // enable tracing (.Off, .Messages, Verbose)
    lc.trace = Trace.Verbose;
    let disposable = lc.start();
    console.log("calling subscription pushh")
    
    // Push the disposable to the context's subscriptions so that the 
    // client can be deactivated on extension deactivation
    context.subscriptions.push(disposable);
}